// ------------------------------------------------------------------------------------
// <copyright file="AutoresBL.cs" company="Universidad Distrital">
// Copyright (c) Universidad Distrital. All rights reserved.
// </copyright>
// <author>Camilo Prieto</author>
// ------------------------------------------------------------------------------------
namespace Biblioteca.Negocio.Clases.BL
{
	using System;
	using System.Collections.Generic;
	using System.Threading.Tasks;
	using Biblioteca.Datos.Clases.DAL.Repositorio;
	using Biblioteca.IC.Acciones.Repositorio;
	using Biblioteca.IC.DTO.Repositorio;
	using Biblioteca.IC.Recursos;
	using TransaccionalNet.EF.Clases;
	using UtilitariosNet.Clases.Comunes.Entidades;
	using Utilitarios.TransversalesNet.Clases.Comunes.Enumeraciones;

	/// <summary>
	/// Clase con las acciones de negocio de la entidad Autores
	/// </summary>
	public class AutoresBL : AccesoComunBL, IAutoresNegocioAccion
	{

		#region ATRIBUTOS
		/// <summary>
		/// Objeto para repositorio Autores
		/// <summary>
		/// Objeto para acciones Autores
		/// </summary>
		private Lazy<IAutoresRepositorioAccion> autoresRepositorioAccion;

		/// <summary>
		/// Objeto para entidad respuesta
		/// </summary>
		Respuesta<IAutoresDTO> respuesta;

		#endregion ATRIBUTOS
		#region CONSTRUCTORES
		/// <summary>
		/// Inicializa una nueva instancia de la clase AutoresBL
		/// </summary>
		/// <param name="argAutoresRepositorioAccion">Acciones entidad Autores</param>
		public AutoresBL(Lazy<IAutoresRepositorioAccion> argAutoresRepositorioAccion = null)
		{
			this.respuesta = new Respuesta<IAutoresDTO>();
			this.autoresRepositorioAccion = argAutoresRepositorioAccion ?? new Lazy<IAutoresRepositorioAccion>(() => new AutoresDAL());
		}

		#endregion CONSTRUCTORES
		#region METODOS PUBLICOS
		/// <summary>
		/// Metodo consultar lista autores
		/// </summary>
		/// <returns>Respuesta tipo Autores </returns>
		public async Task<Respuesta<IAutoresDTO>> ConsultarListaAutoresAsync()
		{

			return await this.EjecutarTransaccionBDAsync<Respuesta<IAutoresDTO>, AutoresBL>(
			System.Transactions.IsolationLevel.ReadUncommitted,
			async () =>
			{
				 respuesta= await this.autoresRepositorioAccion.Value.ConsultarListaAutoresAsync();
				 respuesta.Resultado = true;
				 respuesta.TipoNotificacion = TipoNotificacion.Exitoso;
				 return respuesta;

			});

		}
		/// <summary>
		/// Metodo consultar por llave autores
		/// </summary>
		/// <param name="autores">Entidad a consultar</param>
		/// <returns>Respuesta tipo Autores </returns>
		public async Task<Respuesta<IAutoresDTO>> ConsultarAutoresLlaveAsync(IAutoresDTO autores)
		{
			return await this.EjecutarTransaccionBDAsync<Respuesta<IAutoresDTO>, AutoresBL>(
			System.Transactions.IsolationLevel.ReadUncommitted,
			async () =>
			{
				 respuesta= await this.autoresRepositorioAccion.Value.ConsultarAutoresLlaveAsync(autores);
				 respuesta.Resultado = true;
				 respuesta.TipoNotificacion = TipoNotificacion.Exitoso;
				 return respuesta;

			});
		}

		/// <summary>
		/// Metodo editar autores
		/// </summary>
		/// <param name="autores">Entidad a editar</param>
		/// <returns>Respuesta tipo Autores </returns>
		public async Task<Respuesta<IAutoresDTO>> EditarAutoresAsync(IAutoresDTO autores)
		{
			return await this.EjecutarTransaccionBDAsync<Respuesta<IAutoresDTO>, AutoresBL>(
			System.Transactions.IsolationLevel.ReadUncommitted,
			async () =>
			{
				 respuesta= await this.autoresRepositorioAccion.Value.EditarAutoresAsync(autores);
				 respuesta.Resultado = true;
				 respuesta.TipoNotificacion = TipoNotificacion.Exitoso;
				 respuesta.Mensajes.Add(rcsMensajesComunes.MensajeCreacionEdicionExitosa);
				 return respuesta;

			});
		}


		/// <summary>
		/// Metodo eliminar autores
		/// </summary>
		/// <param name="autores">Entidad a eliminar</param>
		/// <returns>Respuesta tipo Autores </returns>
		public async Task<Respuesta<IAutoresDTO>> EliminarAutoresAsync(IAutoresDTO autores)
		{
			return await this.EjecutarTransaccionBDAsync<Respuesta<IAutoresDTO>, AutoresBL>(
			System.Transactions.IsolationLevel.ReadUncommitted,
			async () =>
			{
				 respuesta= await this.autoresRepositorioAccion.Value.EliminarAutoresAsync(autores);
				 respuesta.Resultado = true;
				 respuesta.TipoNotificacion = TipoNotificacion.Exitoso;
				 respuesta.Mensajes.Add(rcsMensajesComunes.MensajeEntidadEliminadaConExito);
				 return respuesta;

			});
		}

		/// <summary>
		/// Metodo guardar autores
		/// </summary>
		/// <param name="autores">Entidad a guardar</param>
		/// <returns>Respuesta tipo Autores </returns>
		public async Task<Respuesta<IAutoresDTO>> GuardarAutoresAsync(IAutoresDTO autores)
		{
			return await this.EjecutarTransaccionBDAsync<Respuesta<IAutoresDTO>, AutoresBL>(
			System.Transactions.IsolationLevel.ReadUncommitted,
			async () =>
			{
				 respuesta= await this.autoresRepositorioAccion.Value.GuardarAutoresAsync(autores);
				 respuesta.Resultado = true;
				 respuesta.TipoNotificacion = TipoNotificacion.Exitoso;
				 respuesta.Mensajes.Add(rcsMensajesComunes.MensajeCreacionEdicionExitosa);
				 return respuesta;

			});
		}

		/// <summary>
		/// Metodo Guardar lista autores
		/// </summary>
		/// <param name="listaAutores">Lista de entidades a guardar</param>
		/// <returns>Respuesta tipo Autores </returns>
		public async Task<Respuesta<IAutoresDTO>> GuardarListaAutoresAsync(List<IAutoresDTO> listaAutores)
		{
			return await this.EjecutarTransaccionBDAsync<Respuesta<IAutoresDTO>, AutoresBL>(
			System.Transactions.IsolationLevel.ReadUncommitted,
			async () =>
			{
				 respuesta= await this.autoresRepositorioAccion.Value.GuardarListaAutoresAsync(listaAutores);
				 respuesta.Resultado = true;
				 respuesta.TipoNotificacion = TipoNotificacion.Exitoso;
				 respuesta.Mensajes.Add(rcsMensajesComunes.MensajeCreacionEdicionExitosa);
				 return respuesta;

			});
		}
		#endregion METODOS PUBLICOS
		#region METODOS PRIVADOS
		#endregion METODOS PRIVADOS
	}
}

