// ------------------------------------------------------------------------------------
// <copyright file="AutoresController.cs" company="Universidad Distrital">
// Copyright (c) Universidad Distrital. All rights reserved.
// </copyright>
// <author>Camilo Prieto</author>
// ------------------------------------------------------------------------------------
namespace Biblioteca.API.Controllers
{
	using System;
	using System.Collections.Generic;
	using System.Threading.Tasks;
	using Utilitarios.APINet.Clases.Ejecucion;
	using UtilitariosNet.Clases.Comunes.Entidades;
	using UtilitariosNet.Clases.Mapeador.Extensiones;
	using Biblioteca.API.Models.Repositorio;
	using Biblioteca.IC.Acciones.Repositorio;
	using Biblioteca.Negocio.Clases.BL;
	using Microsoft.AspNetCore.Mvc;
	
	/// <summary>
	/// Clase con las capaciades Rest de la entidad Autores
	/// </summary>
	[Route("api/Autores")]
	public class AutoresController : AccesoComunAPI
	{


		/// <summary>
		/// Objeto para negocio Autores
		/// </summary>
		private Lazy<IAutoresNegocioAccion> negocioAutores;

		
		/// <summary>
		/// Inicializa una nueva instancia de la clase AutoresController
		/// </summary>
		public AutoresController()
		{
			this.negocioAutores = new Lazy<IAutoresNegocioAccion>(
										() => new AutoresBL());
		}
		
		/// <summary>
		/// Metodo consultar lista autores
		/// </summary>
		/// <returns>Respuesta tipo Autores </returns>
		[HttpGet]
		[Route("ConsultarListaAutores")]
		public async Task<Respuesta<Autores>> ConsultarListaAutores()
		{
			return await this.EjecutarTransaccionAPI<Task<Respuesta<Autores>>, AutoresController>(async () =>
			{
				return Mapeador.MapearObjetoPorJson<Respuesta<Autores>>(await negocioAutores.Value.ConsultarListaAutoresAsync());
			});
		}
		
		/// <summary>
		/// Metodo consultar por llave autores
		/// </summary>
		/// <param name="autores">Entidad a consultar</param>
		/// <returns>Respuesta tipo Autores </returns>
		[HttpPost]
		[Route("ConsultarAutoresLlave")]
		public async Task<Respuesta<Autores>> ConsultarAutoresLlave(Autores autores)
		{
			return await this.EjecutarTransaccionAPI<Task<Respuesta<Autores>>, AutoresController>(async () =>
			{
				return Mapeador.MapearObjetoPorJson<Respuesta<Autores>>(await negocioAutores.Value.ConsultarAutoresLlaveAsync(autores));
			});
		}
		
		/// <summary>
		/// Metodo guardar autores
		/// </summary>
		/// <param name="autores">Entidad a guardar</param>
		/// <returns>Respuesta tipo Autores </returns>
		[HttpPost]
		[Route("GuardarAutores")]
		public async Task<Respuesta<Autores>> GuardarAutores(Autores autores)
		{
			return await this.EjecutarTransaccionAPI<Task<Respuesta<Autores>>, AutoresController>(async () =>
			{
				return Mapeador.MapearObjetoPorJson<Respuesta<Autores>>(await negocioAutores.Value.GuardarAutoresAsync(autores));
			});
		}
		
		/// <summary>
		/// Metodo editar autores
		/// </summary>
		/// <param name="autores">Entidad a editar</param>
		/// <returns>Respuesta tipo Autores </returns>
		[HttpPut]
		[Route("EditarAutores")]
		public async Task<Respuesta<Autores>> EditarAutores(Autores autores)
		{
			return await this.EjecutarTransaccionAPI<Task<Respuesta<Autores>>, AutoresController>(async () =>
			{
				return Mapeador.MapearObjetoPorJson<Respuesta<Autores>>(await negocioAutores.Value.EditarAutoresAsync(autores));
			});
		}
		
		/// <summary>
		/// Metodo eliminar autores
		/// </summary>
		/// <param name="autores">Entidad a eliminar</param>
		/// <returns>Respuesta tipo Autores </returns>
		[HttpDelete]
		[Route("EliminarAutores")]
		public async Task<Respuesta<Autores>> EliminarAutores(Autores autores)
		{
			return await this.EjecutarTransaccionAPI<Task<Respuesta<Autores>>, AutoresController>(async () =>
			{
				return Mapeador.MapearObjetoPorJson<Respuesta<Autores>>(await negocioAutores.Value.EliminarAutoresAsync(autores));
			});
		}
	}
}

